/* Not all of the listed functions are used in getwaterbalance.
 * Some of the functions are still in test mode. No guarantee is given.
 * Use on personal responsibility only.
 */




#include "filefunc2.h"
#include <math.h>
#include <time.h>
#include "filters.h"

#define ERROR -9.999e14
#define EPSILON 1e-24
#define TWO_PI (6.2831853071795864769252867665590057683943L)

time_t readfiltertime(FILE *fp, struct tm *myTm);

double quadrat(double a);
double absolut(double a);





// entfernt Novalue-Werte aus Daten die bei der Division durch Null entstanden sind (printf-Ausgabe: -nan).
// dies geschieht durch lineare Interpolation wobei angenommen ist, dass die Zeitschritte equidistant sind.
// Dies trifft fìr die gemittelten Werte (average-dateien) aus makeboundaries.c zu.
void clearnovalues(double *data, long lines)
{
	double slope;
	long l, k;
		
	
	for(l=0; l<lines; l++){
		if(isnan(data[l])){
			k=l+1;
			while(isnan(data[k])) k++;
			slope = (data[k]-data[l-1])/(double)(k-(l-1));
			data[l]= data[l-1]+slope; //eigentlich slope*(l-(l-1)) = slope*1
		}
	}
}




/*thresholdfilter filtert sprìnge aus den Daten:
 * Zunñchst werden alle Differenzen der Daten von einem Zeitpunkt zum nñchsten berechnet.
 * Dann werden alle Differenzen auf Null gesetzt, die nicht zwischen thresh_min und thresh_max liegen, (also Sprìnge gelscht).
 * Aus den gebliebenen Differenzen wird der Datenarray neu berechnet.
*/
/*
 thresholdfilter() setzt die Flìsse (deltamass), die die anggebene Schwelle ìberschreiten auf Null und berechenet
 aus den ìbrigbleibenden Flìssen ein neues Massefile. (gut fìr Sickerwasserflìsse, aber Massenerhaltung nicht gewñhrleistet)
 */
			
int thresholdfilter(double **data, double **log, int cols, long lines, double thresh_min, double thresh_max)
{
	double **delta;
	long l;
	int i;
	
	
	/***********************************************
	 * ALLOCATE MEMEORY FOR DELTAS
	/***********************************************/
	delta = dmatrix(cols,lines);
	
	/***********************************************
	 * CALCULATE DELTAS
	/***********************************************/
	for(i=0; i<cols; i++)
		for(l=0; l<(lines-1); l++)
			delta[i][l] = data[i][l+1] - data[i][l];

		
	/***********************************************
	 * SET LOG-ARRAY TO ZERO
	/***********************************************/
	for(i=0; i<cols; i++)
		for(l=0; l<lines; l++)
			log[i][l] = 0.0;
		
	/***********************************************
	 * FILTER DELTAS AND SAVE FILTERED POINTS IN LOG-ARRAY (change units from kg/min to mm/h)
	/***********************************************/
	for(i=0; i<cols; i++)
		for(l=0; l<(lines-1); l++)
			if (delta[i][l] < thresh_min || delta[i][l] > thresh_max){
				log[i][l] = delta[i][l]*60.0*1000.0/998.2;
				delta[i][l]=0.0;
			}
			
	/***********************************************
	 * CALCULATE FILTERED DATA
	/***********************************************/
	for(i=0; i<cols; i++){
		//data[i][0]=0.0;
		for(l=1; l<lines; l++)
			data[i][l] = data[i][l-1] + delta[i][l-1];
	}
		
	return 0;
}


/*
 * wie thresholdfilter, nur sind die Zeitabstñnde nicht zwingend ñquidistant. Flìsse werden durch Zeitdifferenzen berechnet.
 * Zeitdifferenzen mìssen als Sekunden-werte mitgeliefert werden. Die Flussschwellen werden wieder als g/min angegeben.
 */
int thresholdfilter_vartime(double **data, long *t, double **log, int cols, long lines, double thresh_min, double thresh_max)
{
	double **flux;
	long l;
	int i;
	
	
	/***********************************************
	 * ALLOCATE MEMEORY FOR DELTAS
	/***********************************************/
	flux = dmatrix(cols,lines);
	
	/***********************************************
	 * CALCULATE DELTAS
	/***********************************************/
	for(i=0; i<cols; i++)
		for(l=0; l<(lines-1); l++){
			if(t[l+1]==t[l]) flux[i][l]=0.0;
			else flux[i][l] = (data[i][l+1] - data[i][l])/(double)(t[l+1]-t[l])*60.0; //Fluss pro Minute
		}

		
	/***********************************************
	 * SET LOG-ARRAY TO ZERO
	/***********************************************/
	for(i=0; i<cols; i++)
		for(l=0; l<lines; l++)
			log[i][l] = 0.0;
		
	/***********************************************
	 * FILTER DELTAS AND SAVE FILTERED POINTS IN LOG-ARRAY (change units from kg/min to mm/h)
	/***********************************************/
	for(i=0; i<cols; i++)
		for(l=0; l<(lines-1); l++)
			if (flux[i][l] < thresh_min || flux[i][l] > thresh_max){
				log[i][l] = flux[i][l]*60.0*1000.0/998.2;
				flux[i][l]=0.0;
			}
			
	/***********************************************
	 * CALCULATE FILTERED DATA
	/***********************************************/
	for(i=0; i<cols; i++){
		//data[i][0]=0.0;
		for(l=1; l<lines; l++)
			data[i][l] = data[i][l-1] + flux[i][l-1]/60.0*(double)(t[l]-t[l-1]);
	}
		
	return 0;
}

/*
thresholdfilter2() erstzt die Massenpunkte, bei denen der Fluss die Schwelle ìbersteigt durch linear interpolierte Werte.
Vorteil: Massenerhaltung gewñhrleistet, aber Ablassen des SW-Tanks kann nicht richtig berìcksichtigt werden.
*/
int thresholdfilter2(double **data, double **log, int cols, long lines, double thresh_min, double thresh_max)
{
	double delta, slope;
	long l, k, kmin, kmax;
	int i;
	
	/***********************************************
	 * Mark datapoints that are out of tolerance limit with ERROR-code
	/***********************************************/
	k=0;
	for(i=0; i<cols; i++)
		for(l=1; l<lines; l++){
			//Differenz bilden zum letzten Punkt, der nicht als Error deklariert ist
			delta = (data[i][l]-data[i][k])/ (double)(l-k);
							
			if (delta < thresh_min || delta > thresh_max){
				data[i][l] = ERROR;
				log[i][l] = delta*60.0*1000.0/998.2;
			}
			//sonst Koordinate des Punktes in k speichern
			else k=l;
		}
			
	/***********************************************
	 * Replace errorpoints by linear interpolation
	/***********************************************/
	for(i=0; i<cols; i++)
		for(l=1; l<lines; l++){
			if(data[i][l]==ERROR){
				
				kmin=l-1;
				while(data[i][l]==ERROR) l++;
				kmax=l;
				slope = (double)(data[i][kmax]-data[i][kmin])/ (double)(kmax-kmin);
				for(k=kmin+1; k<kmax; k++){
					data[i][k]= data[i][kmin] + (double)(k-kmin)*slope;
				}
			}
		}
			
		
	return 0;
}

/*
 * wie thresholdfilter2, nur sind die Zeitabstñnde nicht zwingend ñquidistant. Flìsse werden durch Zeitdifferenzen berechnet.
 * Zeitdifferenzen mìssen als Sekunden-werte mitgeliefert werden. Die Flussschwellen werden wieder als g/min angegeben.
 */
int thresholdfilter2_vartime(double **data, long *t, double **log, int cols, long lines, double thresh_min, double thresh_max)
{
	double flux, slope;
	long l, k, kmin, kmax;
	int i;
	
	/***********************************************
	 * Mark datapoints that are out of tolerance limit with ERROR-code
	/***********************************************/
	k=0;
	for(i=0; i<cols; i++)
		for(l=1; l<lines; l++){
			//Differenz bilden zum letzten Punkt, der nicht als Error deklariert ist
			flux = (data[i][l]-data[i][k])/ (double)(t[l]-t[k])*60; //Fluss g/min
							
			if (flux < thresh_min || flux > thresh_max){
				//printf("k=%ld - Schwelle ìberschritten: %lf - %lf\n", k, data[i][l], data[i][k]);
				//getch();
				data[i][l] = ERROR;
				log[i][l] = flux*60.0*1000.0/998.2;
			}
			//sonst Koordinate des Punktes in k speichern
			else k=l;
		}
			
	/***********************************************
	 * Replace errorpoints by linear interpolation
	/***********************************************/
	for(i=0; i<cols; i++)
		for(l=1; l<lines; l++){
			if(data[i][l]==ERROR){
				
				kmin=l-1;
				while(data[i][l]==ERROR) l++;
				kmax=l;
				slope = (double)(data[i][kmax]-data[i][kmin])/ (double)(t[kmax]-t[kmin]);
				for(k=kmin+1; k<kmax; k++){
					data[i][k]= data[i][kmin] + (double)(t[k]-t[kmin])*slope;
				}
			}
		}
			
		
	return 0;
}


int runmean(double **dat, int cols, long lines, int n)
{
	int i, k, n1, n2;
	long l;
	double *mean;
	
	/*****************************************
	 * CHECK IF n IS IN VALID RANGE
	/*****************************************/
	
	if(n<1) {
		printf("ERROR IN USING GLIDING MEAN. n is not in the valid range. Only values n>=1 are valid.\n\nABORT");
		exit(1);
	}
	
	if(n>lines){
		printf("ERROR IN USING GLIDING MEAN. n is greater than the number of datapoints\n\nABORT");
		exit(1);
	}
	
	/***************************************
	 * NUMBER OF DATAPOINTS USED FOR GLIDING MEAN IS SEPARATED IN TWO INTERVALS
	 * *************************************/

	//n is odd
	if(n%2) n1=n2=(n-1)/2;
	
	//n is even
	else{
		n1 = n/2 -1; //values before actual datapoint
		n2 = n/2; //values behind actual datapoint
	}
	
	/***************************************
	 * ALLOCATE MEMORY FOR TEMPORARY ARRAY FOR GLIDING MEAN
	 * *************************************/
	mean = (double *)calloc(lines, sizeof(double));
	
	/***************************************
	 * Calculate running mean
	 * *************************************/
	
	for(i=0; i<cols; i++){
		//set mean to zero
		for(l=0; l<lines; l++)
			mean[l]=0.0;
		
		//calculate new mean
			
		for(l=0; l<lines; l++){
			
			//1.case: at the beginning, the running mean can calculated only asymetric from the right side
			if(l<n1){
				for(k=0; k<=l+n2; k++)
					mean[l] += dat[i][k];
				mean[l] = mean[l] /(l+n2+1);
			}
			
			//2.case: at the end, the running mean can calculated only asymetric from the left side
			else if(l>=(lines-n2)){
				for(k=l-n1; k<lines; k++)
					mean[l] += dat[i][k];
				mean[l] = mean[l] /(lines-l+n1);
			}
			
			//3.case: running mean is calculated from both sides symetrically (if n is uneven)
			else{
				for(k=-n1; k<=n2; k++)
					mean[l] += dat[i][l+k];
				
				mean[l] = mean[l]/(n1+n2+1);
			}
		}
		
		//overwrite olddata with gliding mean
		for(l=0; l<lines; l++)
			dat[i][l]=mean[l];
	}
	
	return 0;
}

int runmean(double *dat, long lines, int n)
{
	int i, k, n1, n2;
	long l;
	double *mean;
	
	/*****************************************
	 * CHECK IF n IS IN VALID RANGE
	/*****************************************/
	
	if(n<1) {
		printf("ERROR IN USING GLIDING MEAN. n is not in the valid range. Only values n>=1 are valid.\n\nABORT");
		exit(1);
	}
	
	if(n>lines){
		printf("ERROR IN USING GLIDING MEAN. n is greater than the number of datapoints\n\nABORT");
		exit(1);
	}
	
	/***************************************
	 * NUMBER OF DATAPOINTS USED FOR GLIDING MEAN IS SEPARATED IN TWO INTERVALS
	 * *************************************/

	//n is odd
	if(n%2) n1=n2=(n-1)/2;
	
	//n is even
	else{
		n1 = n/2 -1; //values before actual datapoint
		n2 = n/2; //values behind actual datapoint
	}
	
	/***************************************
	 * ALLOCATE MEMORY FOR TEMPORARY ARRAY FOR GLIDING MEAN
	 * *************************************/
	mean = (double *)calloc(lines, sizeof(double));
	
	/***************************************
	 * Calculate running mean
	 * *************************************/

		
	//calculate new mean
			
	for(l=0; l<lines; l++){
		
		//1.case: at the beginning, the running mean can calculated only asymetric from the right side
		if(l<n1){
			for(k=0; k<=l+n2; k++)
				mean[l] += dat[k];
			mean[l] = mean[l] /(l+n2+1);
		}
		
		//2.case: at the end, the running mean can calculated only asymetric from the left side
		else if(l>=(lines-n2)){
			for(k=l-n1; k<lines; k++)
				mean[l] += dat[k];
			mean[l] = mean[l] /(lines-l+n1);
		}
		
		//3.case: running mean is calculated from both sides symetrically (if n is uneven)
		else{
			for(k=-n1; k<=n2; k++)
				mean[l] += dat[l+k];
			
			mean[l] = mean[l]/(n1+n2+1);
		}
	}
	
	//overwrite olddata with gliding mean
	for(l=0; l<lines; l++)
		dat[l]=mean[l];
	
	return 0;
}


int runmedian(double *dat, long lines, int n)
{
	int i, k, n1, n2, nmedian, p;
	long l;
	double *dataset, *median;
	
	/*****************************************
	 * CHECK IF n IS IN VALID RANGE
	/*****************************************/
	
	if(n<1) {
		printf("ERROR IN USING GLIDING MEAN. n is not in the valid range. Only values n>=1 are valid.\n\nABORT");
		exit(1);
	}
	
	if(n>lines){
		printf("ERROR IN USING GLIDING MEAN. n is greater than the number of datapoints\n\nABORT");
		exit(1);
	}
	
	/***************************************
	 * NUMBER OF DATAPOINTS USED FOR GLIDING MEAN IS SEPARATED IN TWO INTERVALS
	 * *************************************/

	//n is odd
	if(n%2){
		n1=n2=(n-1)/2;
		nmedian = n1+1;
	}
	
	//n is even
	else{
		n1 = n/2 -1; //values before actual datapoint
		n2 = n/2; //values behind actual datapoint
		nmedian = n2;
	}
	
	/***************************************
	 * ALLOCATE MEMORY for data-window and for medianvalues
	 * *************************************/
	dataset = (double *)calloc(n+1, sizeof(double));
	median = (double *)calloc(lines, sizeof(double));
	
	/***************************************
	 * Calculate running median
	 * *************************************/
			
	for(l=0; l<lines; l++){
		
		//1.case: at the beginning, the running mean can calculated only asymetric from the right side
		if(l<n1){
			//copy dataset
			for(k=0; k<=l+n2; k++)
				dataset[k+1] = dat[k];
			
			//sort
			shellsort(l+n2+1, dataset);
			
			//choose median
			p=l+n2+1;
			if(p%2) p=p/2;
			else p=(p+1)/2;
			median[l]= dataset[p];
		}
		
		//2.case: at the end, the running mean can calculated only asymetric from the left side
		else if(l>=(lines-n2)){

			//copy dataset
			for(k=l-n1; k<lines; k++)
				dataset[k-(l-n1)+1] = dat[k];
			
			//sort
			shellsort(lines-(l-n1),dataset);
			
			//choose median
			p=lines-(l-n1);
			if(p%2) p=p/2;
			else p=(p+1)/2;
			median[l]= dataset[p];
		}
		
		//3.case: running median is calculated from both sides symetrically (if n is uneven)
		else{
			//copy dataset
			for(k=-n1; k<=n2; k++)
				dataset[k+n1+1] = dat[l+k];
			
				
			//sort
			shellsort(n,dataset);
			
			//choose median
			median[l]= dataset[nmedian];
							
		}
	}
	
	//overwrite olddata with runmedian
	for(l=0; l<lines; l++)
		dat[l]=median[l];
	
	return 0;
}



int noisethreshold(double **dat, int cols, long lines, double thresh)
{
	int i;
	long l, k, p;
	double diff, slope;
	
	for(i=0; i<cols; i++){
		for(l=0; l<lines;l++){
			
			//look for next value in line k, so that threshhold between value of line l and of line k is exceeded
			for(k=l+1; k<lines; k++){
				diff= (dat[i][l]-dat[i][k]);
				if(diff<0.0) diff=-1*diff;
				if(diff>=thresh) break;
			}
			
			
			//if end of data, interpolate from the last datapoint
			if(k==lines && l<lines-2){
				slope = (dat[i][lines-1]-dat[i][l]) / (lines-1-l);
				for(p=(l+1); p<lines-1;p++)
					dat[i][p] = dat[i][l]+slope*(p-l);
			}
			
			//else interpolation of gap between l and k-1, so that both the k-1)th value and the k)th value are maintained
			else if(k>l+1){
				slope = (dat[i][k-1]-dat[i][l]) / (k-1-l);
				for(p=(l+1); p<(k-1);p++)
					dat[i][p] = dat[i][l]+slope*(p-l);
			}
			
			
			//set linevalue to line k-1 and continue loop with value k
			if(k==lines) l=k;
			else l=k-1;
			
		}
	}
}

int noisethreshold(double *dat, long lines, double thresh)
{
	int i;
	long l, k, p;
	double diff, slope;
	
	for(l=0; l<lines;l++){
		
		//look for next value in line k, so that threshhold between value of line l and of line k is exceeded
		for(k=l+1; k<lines; k++){
			diff= (dat[l]-dat[k]);
			if(diff<0.0) diff=-1*diff;
			if(diff>=thresh) break;
		}
		
		
		//if end of data, interpolate from the last datapoint
		if(k==lines && l<lines-2){
			slope = (dat[lines-1]-dat[l]) / (lines-1-l);
			for(p=(l+1); p<lines-1;p++)
				dat[p] = dat[l]+slope*(p-l);
		}
		
		//else interpolation of gap between l and k-1, so that both the k-1)th value and the k)th value are maintained
		else if(k>l+1){
			slope = (dat[k-1]-dat[l]) / (k-1-l);
			for(p=(l+1); p<(k-1);p++)
				dat[p] = dat[l]+slope*(p-l);
		}
		
		
		//set linevalue to line k-1 and continue loop with value k
		if(k==lines) l=k;
		else l=k-1;
	}
}



int removebadmass(long *tsec, double **mass_Lys, double **mass_SW, int nlys, long lines, FILE *ffilter)
{
	int *filter_Lys, *filter_type, *filter_mode, i;
	long *filter_start, *filter_end, nfilter, f, l;
	time_t now;
	struct tm *myTm;
	double myNan=0.0/0.0;
	
	//INITIALIZE TIME STRUCTURE
	myTm = localtime(&now);
	
	//read number of filter-periods
	fseek(ffilter, 0L, 0);
	nextline(ffilter); //skip header
	nfilter=datalines(ffilter);
	printf("     Filterlines=%ld\n", nfilter);
	
		
	//initialize filterarray
	filter_start= (long*)malloc(nfilter*sizeof(long));
	filter_end= (long*)malloc(nfilter*sizeof(long));
	filter_Lys = (int *)malloc(nfilter*sizeof(int));
	filter_type = (int *)malloc(nfilter*sizeof(int)); //0=Sickerwasser, 1=Lysimeter, 2=beide
	
	
	//read filterdata
	fseek(ffilter, 0L, 0);
	nextline(ffilter);
	
	for(f=0; f<nfilter; f++){
		
		fscanf(ffilter, "%d", &filter_Lys[f]);
		fscanf(ffilter, "%d", &filter_type[f]);

		filter_start[f] = readfiltertime(ffilter, myTm);
		fgetc(ffilter);
		
		filter_end[f] = readfiltertime(ffilter, myTm);		
		nextline(ffilter);
		
		//add 1 hour for MEZ
		filter_start[f] += 3600;
		filter_end[f] += 3600;
		
		//printf("f=%ld:\tLys=%d\ttpye=%d\tstart=%.3ld\tend=%.3ld\n",f, filter_Lys[f], filter_type[f], filter_start[f], filter_end[f]);
		//getch();
	}
	
	/**********************************
	 * Filtering data
	 * ********************************/
	//start Loop over filterperiods
	for(f=0; f<nfilter; f++){
			
		//search start time
		l=0;
		while(tsec[l]<filter_start[f] && l<lines)
			l++;
		
		//remove bad data until end time
		while(tsec[l]<=filter_end[f] && l<lines){
			
			//1. Filter is valid for all lysimeter numbers
			if(filter_Lys[f]==7){
				for(i=0; i<nlys; i++){
					//filter seepage data
					if(filter_type[f]==0 || filter_type[f]==2)
						//mass_SW[i][l]=ERROR;
						mass_SW[i][l]=myNan;
					//filter lysimeter data
					if(filter_type[f]==1 || filter_type[f]==2)
						//mass_Lys[i][l]=ERROR;
						mass_Lys[i][l]=myNan;
				}
			}
			
			//2. filter only special lysimeter
			else{
				i=filter_Lys[f]-1;
				//filter seepage data
				if(filter_type[f]==0 || filter_type[f]==2)
					//mass_SW[i][l]=ERROR;
						mass_SW[i][l]=myNan;
				//filter lysimeter data
				if(filter_type[f]==1 || filter_type[f]==2)
					//mass_Lys[i][l]=ERROR;
						mass_Lys[i][l]=myNan;
			}					
				
			l++;
		}
	}
	
	for(i=0; i<nlys; i++){
		clearnovalues(mass_Lys[i], lines);
		clearnovalues(mass_SW[i], lines);
	}
		
	
	return 0;
}

int removebadmass2(long *tsec, double **mass_Lys, double **mass_SW, int nlys, long lines, FILE *ffilter)
{
	int *filter_Lys, *filter_type, *filter_mode, i;
	long *filter_start, *filter_end, nfilter, f, l;
	long start, end;
	double *dm ,offset;
	char buffer[20];
	time_t now;
	struct tm *myTm;
	
	//INITIALIZE TIME STRUCTURE
	myTm = localtime(&now);
	
	//read number of filter-periods
	fseek(ffilter, 0L, 0);
	nextline(ffilter); //skip header
	nfilter=datalinesUnix(ffilter);
	printf("     Filterlines=%ld\n", nfilter);
		
	//initialize filterarray
	filter_start= (long*)malloc(nfilter*sizeof(long));
	filter_end= (long*)malloc(nfilter*sizeof(long));
	filter_Lys = (int *)malloc(nfilter*sizeof(int));
	filter_type = (int *)malloc(nfilter*sizeof(int)); //0=Sickerwasser, 1=Lysimeter, 2=beide
	filter_mode = (int *)malloc(nfilter*sizeof(int)); //0=points, 1=step
	dm = (double *)malloc(lines*sizeof(double));
	
	
	//read filterdata
	fseek(ffilter, 0L, 0);
	nextline(ffilter);
	
	for(f=0; f<nfilter; f++){
		
		fscanf(ffilter, "%d", &filter_Lys[f]);
		fscanf(ffilter, "%d", &filter_type[f]);

		filter_start[f] = readfiltertime(ffilter, myTm);
		fgetc(ffilter);
		
		filter_end[f] = readfiltertime(ffilter, myTm);
		fgetc(ffilter);
		
		fscanf(ffilter, "%s", buffer);
		if (strcmp(buffer, (char *)"points") == 0)
			filter_mode[f] = 0;
		
		if (strcmp(buffer, (char *)"step") == 0)
			filter_mode[f] = 1;
		
		nextline(ffilter);
		
		//printf("f=%ld:\tLys=%d\ttpye=%d\tstart=%.3ld\tend=%.3ld\n",f, filter_Lys[f], filter_type[f], filter_start[f], filter_end[f]);
		//getch();
	}
	
	/**********************************
	 * Filtering data
	 * ********************************/
	//start Loop over filterperiods
	for(f=0; f<nfilter; f++){
			
		if(filter_mode[f]==0)
		{
			//search start time
			l=0;
			while(tsec[l]<filter_start[f] && l<lines)
				l++;
			
			//remove bad data until end time
			while(tsec[l]<=filter_end[f] && l<lines){
				
				//1. Filter is valid for all lysimeter numbers
				if(filter_Lys[f]==7){
					for(i=0; i<nlys; i++){
						//filter seepage data
						if(filter_type[f]==0 || filter_type[f]==2)
							mass_SW[i][l]=ERROR;
						//filter lysimeter data
						if(filter_type[f]==1 || filter_type[f]==2)
							mass_Lys[i][l]=ERROR;
					}
				}
				
				//2. filter only special lysimeter
				else{
					i=filter_Lys[f]-1;
					//filter seepage data
					if(filter_type[f]==0 || filter_type[f]==2)
						mass_SW[i][l]=ERROR;
					//filter lysimeter data
					if(filter_type[f]==1 || filter_type[f]==2)
						mass_Lys[i][l]=ERROR;
				}					
					
				l++;
			}
		}
		
		if(filter_mode[f]==1)
		{
			if(filter_Lys[f]==7){
				printf("\n\nFEHLER im FIlterfile!!!\nManueller-Steps-Filter geht nur fìr einzelne Lysimeter, nicht fìr alle gleichzeitig.\n Entfernen Sie die 7 im Filterfile_mass und geben Sie dafìr die Nummer des jew. Lysimeters an.\n\n");
			}
			
			else{
				i=filter_Lys[f]-1;
		
				//filter seepage data
				if(filter_type[f]==0 || filter_type[f]==2){
					
					//build differences
					for(l=0; l<lines-1; l++)
						dm[l]=mass_SW[i][l+1]-mass_SW[i][l];
					
// 					
					
					//search start time
					l=0;
					while(tsec[l]<filter_start[f] && l<lines)
						l++;
					start=l;
					
					
					//search end time
					while(tsec[l]<=filter_end[f] && l<lines){
						l++;
					}
					end=l;
					
					//remove mass changes in timeinterval
					for(l=start+1; l<end; l++)
						mass_SW[i][l]=mass_SW[i][start];
					
					//substract offset in the following
					offset=mass_SW[i][end]-mass_SW[i][start];
					for(l=end; l<lines; l++)
						mass_SW[i][l]-=offset;
				}
				
				//filter lysimeter data
				if(filter_type[f]==1 || filter_type[f]==2){
										
					//build differences
					for(l=0; l<lines-1; l++)
						dm[l]=mass_Lys[i][l+1]-mass_Lys[i][l];
					
// 					
					
					//search start time
					l=0;
					while(tsec[l]<filter_start[f] && l<lines)
						l++;
					start=l;
					
					
					//search end time
					while(tsec[l]<=filter_end[f] && l<lines){
						l++;
					}
					end=l;
					
					//remove mass changes in timeinterval
					for(l=start+1; l<end; l++)
						mass_Lys[i][l]=mass_Lys[i][start];
					
					//substract offset in the following
					offset=mass_Lys[i][end]-mass_Lys[i][start];
					for(l=end; l<lines; l++)
						mass_Lys[i][l]-=offset;
				}
					
			}
		}
				
				
		
	}
	
	return 0;
}

time_t readfiltertime(FILE *fp, struct tm *myTm)
{
	char buffer[127], delimiter[] = "/.:,_", *ptr;
	

	fscanf(fp,"%s",buffer);
		
	ptr = strtok(buffer, delimiter); myTm->tm_year=(atoi(ptr)-1900);//Jahr
	//printf("\nJahr: %s  %d",ptr,atoi(ptr));
	ptr = strtok(NULL, delimiter); myTm->tm_mon=(atoi(ptr)-1);//Monat
	//printf("\nMonat: %s  %d",ptr,atoi(ptr));
	ptr = strtok(NULL, delimiter); myTm->tm_mday=atoi(ptr);//Tag
	//printf("\nTag: %s  %d",ptr,atoi(ptr));
	ptr = strtok(NULL, delimiter); myTm->tm_hour=atoi(ptr);//Stunde
	//printf("\nStunde: %s  %d",ptr,atoi(ptr));
	ptr = strtok(NULL, delimiter); myTm->tm_min=atoi(ptr); //Minute
	//printf("\nMinute: %s  %d",ptr,atoi(ptr));
	myTm->tm_sec=0;
	
	
	//summertime-flag is set to 0 - d.h. Sommerzeit-Korrektur wird nicht durchgefìhrt
	myTm->tm_isdst=0;
	
	//add 1 hour for MEZ
	return mktime(myTm);
}





int synchrothreshold(double **mass, long lines, int cols, double *thresh)
{
	int i, actualwidth, windowwidth=0, windowmin, windowmax, reach;
	long l, k;
	double min, max, temp, **flux, *accuracy, **mass_cpy;
	double minthresh=0.01, maxthresh=0.200, actthresh, step=0.01, min_accuracy=0;
	
	
	
	//Speicher bereitstellen
	flux = dmatrix(cols,lines);
	accuracy = (double *) calloc(lines,sizeof(double));
	mass_cpy = dmatrix(cols, lines);
			
	//Schwellenwert mit minthresh initialisieren
	for(l=0; l<lines;l++)
		thresh[l]=minthresh;
		//thresh[l]=0.0;
	
	
	reach=0;
	while(reach==0){	
		
		//Iterationsflag zurìcksetzen
		reach=1;		
				
		//Aktuellen Schwellenwert erhhen
		actthresh+=step;
		fprintf(stderr,"thresh=%lf\n", actthresh);
		
		//Kopie von Masse erstellen
		for(i=0; i<cols; i++)
			for(l=1; l<lines;l++)
				mass_cpy[i][l]=mass[i][l];
			
		//Flìsse fìr MinThresh berechnen
		for(i=0; i<cols; i++)
			noisethreshold_variabel(mass_cpy[i], lines, cols, thresh, flux[i]);
		
		//Genauigkeit fìr alle Punkte bestimmen (=Abweichung zwischen Lysimetern)
		for(l=0; l<lines; l++){
			//kleinsten u grßten Niederschlag finden
			min=flux[0][l];
			max=flux[0][l];
			for(i=1; i<cols; i++){
				if(flux[i][l]<min) min=flux[i][l];
				if(flux[i][l]>max) max=flux[i][l];
			}
			//Abweichung berechnen
				// 		if (min<0){
				// 			temp=min; min=max; max=temp;
				// 		}			
			if(max<=0) accuracy[l]=1;	//Wenn alle Lysimeter ET anzeigen, wird Schwelle nicht erhht!
				else accuracy[l]=min/max;
				   
			//Ist Abweichung an irgendeiner Stelle zu groß, Iterationsflag wieder auf 0 setzen und Schwellenwert hier hochsetzen.
			if(accuracy[l]<min_accuracy){
				reach=0;
				windowmin=l-windowwidth;
				if(windowmin<0) windowmin=0;
				windowmax=l+windowwidth;
				if(windowmax<=lines) windowmax=lines-1;
				for(k=windowmin;k<=windowmax;k++)
					thresh[k]=actthresh;
			}
		}
		
		//Wenn maximale Schwelle schon erreicht ist, Iterationsflag auf "erreicht" setzen
		if(actthresh>=maxthresh)
			reach=1;
	}
	
	//Abschließendes Filtern
	for(l=0; l<lines;l++)
		if(thresh[l]==0) thresh[l]=minthresh;
	for(i=0; i<cols; i++)
			noisethreshold_variabel(mass[i], lines, cols, thresh, flux[i]);
		

}

int synchrothreshold_fiveLysimeters(double **mass, long lines, int cols, double *thresh)
{
	int i, actualwidth, reach, positives, negatives, accordance;
	long l, k;
	double min, max, temp, **flux, **mass_cpy;
	double minthresh=0.01, maxthresh=0.099, actthresh, step=0.01, min_accuracy=0;
	
	
	
	//Speicher bereitstellen
	flux = dmatrix(cols,lines);
	mass_cpy = dmatrix(cols, lines);
			
	//Schwellenwert mit minthresh initialisieren
	for(l=0; l<lines;l++)
		thresh[l]=minthresh;
		//thresh[l]=0.0;
	
	
	reach=0;
	while(reach==0){	
		
		//Iterationsflag zurìcksetzen
		reach=1;		
				
		//Aktuellen Schwellenwert erhhen
		actthresh+=step;
		fprintf(stderr,"thresh=%lf\n", actthresh);
		
		//Kopie von Masse erstellen
		for(i=0; i<cols; i++)
			for(l=1; l<lines;l++)
				mass_cpy[i][l]=mass[i][l];
			
		//Flìsse fìr MinThresh berechnen
		for(i=0; i<cols; i++)
			noisethreshold_variabel(mass_cpy[i], lines, cols, thresh, flux[i]);
		
		/*//Funktionstest
		for(l=13500; l<13700 && l<lines; l++){
			printf("%ld\t",l+1);
			for(i=0;i<6; i++)
				printf("%lf\t",flux[i][l]*1000.0/998.2*60.0);
			printf("\n");
		}
		printf("\n\n");*/
		
		//Genauigkeit fìr alle Punkte bestimmen (=Abweichung zwischen Lysimetern)
		for(l=0; l<lines; l++){
			//kleinsten u grßten Niederschlag finden
			positives=0;
			negatives=0;
			for(i=1; i<cols; i++){
				if(flux[i][l]>=0.0) positives++;
				if(flux[i][l]<=0.0) negatives++;
			}
			//Abweichung berechnen
				// 		if (min<0){
				// 			temp=min; min=max; max=temp;
				// 		}			
			if(positives>=5 || negatives>=5) accordance=1;	//Wenn alle Lysimeter ET anzeigen, wird Schwelle nicht erhht!
				else accordance=0;
				   
			//Ist Abweichung an irgendeiner Stelle zu groß, Iterationsflag wieder auf 0 setzen und Schwellenwert hier hochsetzen.
			if(!accordance){
				reach=0;
				thresh[l]=actthresh;
			}
		}
		
		//Wenn maximale Schwelle schon erreicht ist, Iterationsflag auf "erreicht" setzen
		if(actthresh>maxthresh){
			reach=1;
		}
	}
	
	/* Funktionstest
	for(l=0; l<lines; l++){
		printf("%ld\t%lf\n",l+1, thresh[l]);
	}*/
	
	//Abschließendes Filtern
	for(l=0; l<lines;l++)
		if(thresh[l]==0) thresh[l]=minthresh;
	for(i=0; i<cols; i++)
			noisethreshold_variabel(mass[i], lines, cols, thresh, flux[i]);
		
				
}


void noisethreshold_variabel(double *mass, long lines, int cols, double *thresh, double *flux)
{
	long l;
	double diff;
	bool *noval;
	
	//Indikator-array fìr zu filternde Bereiche: Speicher bereitstellen
	noval = (bool *)calloc(lines, sizeof(bool));
		
	//Massdatei filtern	
	for(l=1; l<lines;l++){
		diff=mass[l]-mass[l-1];
		if(diff<0) diff*=-1;
		if(diff<thresh[l]){
			mass[l]=mass[l-1];
			noval[l]=1;
		}
	}
			
	//gefilterte Bereiche linear interpolieren
	for(l=0; l<lines-1;l++)
		if(noval[l]==1) mass[l]=NAN;
	clearnovalues(mass, lines);
		
	//Flìsse neu berechnen	
	for(l=1; l<lines; l++)
		flux[l]=mass[l]-mass[l-1];
}
		
			
	
void noisethreshold_variabel_ohneInterpol(double *mass, long lines, int cols, double *thresh, double *flux)
{
	long l;
	double diff;
	bool *noval;
	
	//Indikator-array fìr zu filternde Bereiche: Speicher bereitstellen
	noval = (bool *)calloc(lines, sizeof(bool));
		
	//Massdatei filtern	
	for(l=1; l<lines;l++){
		diff=mass[l]-mass[l-1];
		if(diff<0) diff*=-1;
		if(diff<thresh[l]){
			mass[l]=mass[l-1];
			noval[l]=1;
		}
	}
			
}


void shellsort(int n, double a[])
// Sorts an array a[] into ascending numerical order by Shells method (diminishing increment
// sort). a is replaced on output by its sorted rearrangement. Normally, the argument n should
// be set to the size of array a, but if n is smaller than this, then only the first n elements of a
// are sorted. This feature is used in selip.
// Sortier-Alg. ist schnell genug fìr n<50, sonst lieber quicksort nehmen!!
{
	int i,j,inc;
	double v;
	inc=1;			// Determine the starting increment.
	do {
		inc *= 3;
		inc++;
	} while (inc <= n);
	do {				// Loop over the partial sorts.
		inc /= 3;
		for (i=inc+1;i<=n;i++) { 	//Outer loop of straight insertion.
			v=a[i];
			j=i;
			while (a[j-inc] > v) {		// Inner loop of straight insertion.
				a[j]=a[j-inc];
				j -= inc;
				if (j <= inc) break;
			}
			a[j]=v;
		}
	} while (inc > 1);
}
	


double quadrat(double a)
{
	return a*a;
}

double absolut(double a)
{
	if(a<0) return -a;
	else return a;
}
	
	
	
		