/*
g++ getwaterbalance.c filters.c filefunc2.c -o getwaterbalance
*/

#include <time.h>
#include "filefunc2.h"
#include "filters.h"

#define ERROR -9.999e14
#define MINGAP 10

time_t readtime(char *buffer, struct tm *myTm);
void getfluxes(long lines, int cols, double **mass_Lys, double **mass_SW, double timestep, double **topflux, double **bottomflux);
void getETPSS(long lines, int cols, double **mass_sum, double **mass_SW, double **evapo, double **precip, double **storage, double **seepage);
void getdailyprecip(long lines, int nlys, char **date, double **precip , FILE *fout);
void readmassdata(FILE *data, long lines, int nlys, int ntimecols, long *tsec, double **mass_Lys, double **mass_SW);
int printdataNN(FILE *fp, long *tsec, double **data, long cols, long lines); //um die negativen Massen des manuellen Filters als NaN zu schreiben
int findkey_int(FILE *param, long lines, char *keyword);
double findkey(FILE *param, long lines, char *keyword);

int main(int argc, char *arg_v[])
{
	int ntimecols, nlys, verbose, runmean_n, median_n, synchro;
	int i, timestep, printinter, printlog, eventbasedtest;
	long lines, l, datapoints, k, *tsec, *tsec_orig;
	char filename[256], buffer[256], **date_orig, **date, c, *ptr, command[256], folder[256];
	FILE *param, *data, *fout, *ffilter, *fthresh, *fout2;
	double  autofil_SW_max, autofil_SW_min, autofil_Lys_max, autofil_Lys_min, noisethresh;
	double **mass_SW_orig, **mass_Lys_orig, **mass_sum_orig, *time_orig;
	double **mass_SW, **mass_Lys, **mass_sum, *time, **logdata, **topflux, **bottomflux;
	double ratio, start, end, between, tdays;
 	double **evapo, **precip, **storage, **seepage;
	double diff_timestep;
	double *thresh;
	
	
	/**********************************************
	 * Open parameter-File
	/**********************************************/
	sprintf(filename, "param.dat");
	if((param = fopen(filename,"r"))==NULL)
	{
		printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename);
		exit(1);
	}
	
	
	/**********************************************
	 * GET PARAMETERS OF THE INPUTDATASET
	/**********************************************/
	lines=datalines(param);
	ntimecols=findkey_int(param, lines, (char*)"ntimecols");
	nlys=findkey_int(param, lines,(char*)"nlys");


	/**********************************************
	 * GET FILTERING PARAMETERS
	/**********************************************/
	//local threshold
	autofil_SW_max  = findkey(param, lines, (char*)"maxSPF");	//g/min
	autofil_SW_min  = findkey(param, lines, (char*)"minSPF");	//g/min
	autofil_Lys_max = findkey(param, lines, (char*)"maxLF");	//g/min
	autofil_Lys_min = findkey(param, lines, (char*)"minLF");	//g/min
	
	//median and runmean
	median_n =  findkey_int(param, lines, (char*)"median_n");	//min
	runmean_n = findkey_int(param, lines, (char*)"runmean_n");	//min
	
	//oscillation threshold	
	noisethresh = findkey(param, lines, (char*)"noisethresh");	//kg
		
	//adaptive threshold using parallel lysimeters
	synchro = findkey_int(param, lines, (char*)"synchro");
	//0 = normaler Noisethreshold ohne synchro, 1 = SynchroFilter an, 2 = SynchroFilter_5Lysimeters an, 3 = alter SynchroFilter
	
	
	/**********************************************
	 * GET PARAMETERS FOR OUTPUT
	/**********************************************/
	verbose = findkey_int(param, lines, (char*)"verbose");	//give output to terminal, while calculating
	timestep = 1;	//min,	# number of minutes between each timestep in the calculated data array (and the output-file)
	printinter = findkey_int(param, lines, (char*)"printinter");	//print intermediate results (0=no, 1=yes)
	printlog = findkey_int(param, lines, (char*)"printlog");

	printf("Read parameters:\nntimecols=%d\tnlys=%d\nAutomatic Threshold\n", ntimecols, nlys);
	printf("maxSPF=%lf\tminSPF=%lf\nmaxLF=%lf\tminLF=%lf\n", autofil_SW_max, autofil_SW_min , autofil_Lys_max, autofil_Lys_min);
	printf("Median and moving mean:\nmedian_n=%d\nrunmean_n=%d\n",median_n, runmean_n);
	if (synchro==0)
		printf("Oscillation threshold:\nnoisethresh=%lf\n", noisethresh);
	if (synchro>0)
		printf("ATTENTION: Synchro-Filter is activated with synchro=%d\n", synchro);
	
	
	
	
	/**********************************************
	 * GET INPUT PARAMETERS: INPUTFILE
	/**********************************************/
	
	if (argc != 1)
	{
		printf("\nUSAGE ERROR\n");
		printf("\nUSAGE: ./_getwaterbalance\n");
		return 0;
	}
	
	
	
	/**********************************
	* OPEN Data File
	* ********************************/
	
	sprintf(filename, "%s", (char *)"input/mass.dat");
	if((data = fopen(filename,"r"))==NULL)
	{
		printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename);
		exit(1);
	}

	/**********************************
	* GET NUMBER OF DATALINES
	* ********************************/
	
	nextline(data); // skip header
	lines=datalines(data);
	if (verbose) printf("\nnumber of datalines=%ld\n",lines);
	
	/**********************************************
	 * ALLOCATE MEMORY for the original data in the file (with original timesteps)
	/**********************************************/
	
	mass_Lys_orig = dmatrix(nlys,lines);
	mass_SW_orig  = dmatrix(nlys, lines);
	mass_sum_orig  = dmatrix(nlys, lines);
	logdata  = dmatrix(nlys, lines);
	time_orig = (double *)calloc(lines, sizeof(double));
	date_orig = chmatrix(lines, 50);
	tsec_orig = (long *)calloc(lines, sizeof(long));
	topflux  = dmatrix(nlys, lines);
	bottomflux  = dmatrix(nlys, lines);
		
	
	
	/**********************************
	* READ DATA
	* ********************************/	
	readmassdata(data, lines, nlys, ntimecols, tsec_orig, mass_Lys_orig, mass_SW_orig); 
	
	//calculate mass-sum
	for(i=0; i<nlys; i++)
		for(l=0; l<lines; l++)
			mass_sum_orig[i][l] = mass_Lys_orig[i][l] + mass_SW_orig[i][l];
	

	/*************************************
	 * CHECK IF TIMES ARE IN THE RIGHT ORDER
	 * **********************************/
	for(l=0; l<lines-1; l++){
		if(tsec_orig[l+1]<tsec_orig[l]){
			printf("\n\nTIMES ARE NOT IN RIGHT ORDER (line %ld and %ld)\n\nABORT PROCESS\n\n", l+2, l+3);
			exit(1);
		}
		if(tsec_orig[l+1]==tsec_orig[l]){
			printf("\n\nEQUAL TIMESTEPS IN DIFFERENT LINES: (line %ld and %ld)\n\nSTAY ALERT\n\n", l+2, l+3);
		}
		
		//remark:
		//if to timesteps are equal, in the following, when calculating the new equal timesteps
		//the program may use the first time for interpolating values before, and the second time for calculating values after that time
	}
	
	/*************************************
	 * Create output (seperate mass files for Lysimeters and Seepage tanks, and flux file)
	 * **********************************/
	
	//******** print outputfile *************
	if(1){
		//massfiles
		sprintf(filename, (char *)"output/00_signal_Sw.dat");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		
		printdatatofile(fout, tsec_orig, mass_SW_orig, nlys, lines);
		fclose(fout);
		
		sprintf(filename, (char *)"output/00_signal_Lys.dat");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		
		printdatatofile(fout, tsec_orig, mass_Lys_orig, nlys, lines);
		fclose(fout);
		
		sprintf(filename, (char *)"output/00_signal_sum.dat");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		
		printdatatofile(fout, tsec_orig, mass_sum_orig, nlys, lines);
		fclose(fout);
		
		//fluxfiles
		getfluxes(lines, nlys, mass_sum_orig, mass_SW_orig, timestep, topflux, bottomflux);
				
		sprintf(filename, (char *)"output/00_signal_topflux.dat");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		printdatatofile(fout, tsec_orig, topflux, nlys, lines);
		fclose(fout);
		
		sprintf(filename, (char *)"output/00_signal_bottomflux.dat");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		printdatatofile(fout, tsec_orig, bottomflux, nlys, lines);
		fclose(fout);
		
	}
	
	/****************************************
	 * MANUAL FILTERING WITH FILTERING FILE
	/****************************************/
	printf("Manual filtering...\n");
	
	
	//OPEN FILTERFILE and filter when existing!
	
	sprintf(filename, (char *)"input/filterfile_mass.dat");
	if((ffilter = fopen(filename, "r"))==NULL)
		printf("\n\nDatei %s kann nicht geffnet werden!\nSkip manual filtering...\n\n", filename);
	else
		removebadmass(tsec_orig, mass_Lys_orig, mass_SW_orig, nlys, lines, ffilter);
	
	
	//calculate mass-sum
	for(i=0; i<nlys; i++)
		for(l=0; l<lines; l++)
			mass_sum_orig[i][l] = mass_Lys_orig[i][l] + mass_SW_orig[i][l];
		
		
	//******** print outputfile *************
	if(printinter){
		//masses
		sprintf(filename, (char *)"output/01_manual_Sw.dat");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		
		printdataNN(fout, tsec_orig, mass_SW_orig, nlys, lines);
		fclose(fout);
		
		sprintf(filename, (char *)"output/01_manual_Lys.dat");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		
		printdataNN(fout, tsec_orig, mass_Lys_orig, nlys, lines);
		fclose(fout);
		
		sprintf(filename, (char *)"output/01_manual_sum.dat");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		
		printdataNN(fout, tsec_orig, mass_sum_orig, nlys, lines);
		fclose(fout);
		
		//fluxfiles
		getfluxes(lines, nlys, mass_sum_orig, mass_SW_orig, timestep, topflux, bottomflux);
				
		sprintf(filename, (char *)"output/01_manual_topflux.dat");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		printdatatofile(fout, tsec_orig, topflux, nlys, lines);
		fclose(fout);
		
		sprintf(filename, (char *)"output/01_manual_bottomflux.dat");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		printdatatofile(fout, tsec_orig, bottomflux, nlys, lines);
		fclose(fout);
	}
	
	/************************************
	 * Local threshold-filter
	/************************************/
	printf("Thresholdfilter...\n");
	
	//FILTER Seepage-mass (Filter-Thresholds are changed from gramm/timestep to kilogramm/min)
	thresholdfilter_vartime(mass_SW_orig, tsec_orig, logdata, nlys, lines, autofil_SW_min/1000, autofil_SW_max/1000);
	
	
	//******** print logfiles *************
	if(printinter){
		//sprintf(filename, "%s/autofilter_Sw_all.log", folder);
		sprintf(filename, (char *)"output/02_logfile_thresh_Sw_all.log");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		
		printdatatofile(fout, tsec_orig, logdata, nlys, lines);
		fclose(fout);
		
		sprintf(filename, (char *)"output/02_logfile_thresh_Sw_points.log");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		
		printnonzerotofile(fout, tsec_orig, logdata, nlys, lines);
		fclose(fout);
	}
	
	//calculate mass-sum
	for(i=0; i<nlys; i++)
		for(l=0; l<lines; l++)
			mass_sum_orig[i][l] = mass_Lys_orig[i][l] + mass_SW_orig[i][l];
		
	//FILTER LYSIMETER-mass (Filter-Thresholds are changed from gramm/timestep to kilogramm/min)
	thresholdfilter2_vartime(mass_sum_orig, tsec_orig, logdata, nlys, lines, autofil_Lys_min/1000, autofil_Lys_max/1000);
	
	
	//******** print logfiles *************
	if(printlog){
		sprintf(filename, (char *)"output/02_logfile_thresh_Lys_all.log");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}

		printdatatofile(fout, tsec_orig, logdata, nlys, lines);
		fclose(fout);
		
		sprintf(filename, (char *)"output/02_logfile_thresh_Lys_points.log");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		
		printnonzerotofile(fout, tsec_orig, logdata, nlys, lines);
		fclose(fout);
	}
	
	
	
	
		
	//******** print outputfile *************
	if(printinter){
		//print mass files
		sprintf(filename, (char *)"output/02_threshold_Sw.dat");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		
		printdatatofile(fout, tsec_orig, mass_SW_orig, nlys, lines);
		fclose(fout);
		
// 		sprintf(filename, (char *)"output/02_threshold_Lys.dat");
// 		fout = fopen(filename, "w");
// 		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
// 		
// 		printdatatofile(fout, tsec_orig, mass_Lys_orig, nlys, lines);
// 		fclose(fout);
		
		sprintf(filename, (char *)"output/02_threshold_sum.dat");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		
		printdatatofile(fout, tsec_orig, mass_sum_orig, nlys, lines);
		fclose(fout);
		
		//generate and print fluxfiles
		getfluxes(lines, nlys, mass_sum_orig, mass_SW_orig, timestep, topflux, bottomflux);
				
		sprintf(filename, (char *)"output/02_threshold_topflux.dat");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		printdatatofile(fout, tsec_orig, topflux, nlys, lines);
		fclose(fout);
		
		sprintf(filename, (char *)"output/02_threshold_bottomflux.dat");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		printdatatofile(fout, tsec_orig, bottomflux, nlys, lines);
		fclose(fout);
	}
	

	/*************************************
	 * ALLOCATE MEMORY FOR THE EQUAL TIMESTEP_DATA
	 * **********************************/
	
	
	//calculate number of datapoints
	datapoints = (long)((double)(tsec_orig[lines-1] - tsec_orig[0]) / (timestep*60.));
	datapoints += 1; //for first datapoint
	
	if(verbose) printf("\nCalculated datapoints: %ld\n\n", datapoints);
	
	mass_Lys = dmatrix(nlys, datapoints);
	mass_SW  = dmatrix(nlys, datapoints);
	time     = (double *)calloc(datapoints, sizeof(double));
	date     = chmatrix(datapoints, 50);
	tsec     = (long *)calloc(datapoints, sizeof(long));
	
	evapo    = dmatrix(nlys, datapoints);
	precip   = dmatrix(nlys, datapoints);
	storage  = dmatrix(nlys, datapoints);
	seepage  = dmatrix(nlys, datapoints);
	mass_sum = dmatrix(nlys, datapoints);
	thresh   = (double *)calloc(datapoints,sizeof(double));
	
	free(topflux);
	free(bottomflux);
	//free(logdata);
	
	//logdata  = dmatrix(nlys, datapoints);
	topflux  = dmatrix(nlys, datapoints);
	bottomflux  = dmatrix(nlys, datapoints);
	
	
	/*************************************
	 * CALCULATE EQUAL TIMESTEP_DATA
	 * **********************************/
	printf("Calculate equal timestep-data...\n");
	
	tsec[0]=tsec_orig[0];
	for(l=1;l<datapoints;l++){
		tsec[l] = tsec[l-1]+timestep*60; //each one timestep further
	}
	
	/************************************
	 * CREATE DATE FOR THE NEW TIMESTEPS
	 * /************************************/
	getdate(datapoints, tsec, date);
	 
	/************************************
	 * INTERPOLATE DATAPOINTS FOR THE NEW TIMESTEPS
	/************************************/
	
	// FIRST TIMES ARE EQUAL; SO JUST COPY DATA FOR THE FIRST DATAPOINT
	for(i=0; i<nlys; i++){
		mass_sum[i][0] = mass_sum_orig[i][0];
		mass_SW[i][0] = mass_SW_orig[i][0];
	}
	
	//SET COUNTER TO ZERO
	k=0;
	for(l=1; l<datapoints; l++){
		
		//look for the two times in the orignial data sets, so that the first (k-1) is lower and the second (k)greater equal
		while(tsec_orig[k]<tsec[l]) k++;
		
		//if the greater time is equal, just copy data
		if (tsec_orig[k]==tsec[l]){
			for(i=0; i<nlys; i++){
				mass_sum[i][l] = mass_sum_orig[i][k];
				mass_SW[i][l] = mass_SW_orig[i][k];
			}
		}
		//else interpolate
		else{
			start = tsec_orig[k-1];
			end = tsec_orig[k];
			between = tsec[l];
			ratio = (double)(between-start)/(end-start);
			
			for(i=0; i<nlys; i++){
				mass_sum[i][l] = mass_sum_orig[i][k-1] + ratio * (mass_sum_orig[i][k]-mass_sum_orig[i][k-1]);
				mass_SW[i][l] = mass_SW_orig[i][k-1] + ratio * (mass_SW_orig[i][k]-mass_SW_orig[i][k-1]);
			}
		}
			
		//set counter one back, so that tsec_orig[k] < tsec[l]
		k--; 
	}
	
	/************************************
	 * Free space of old data-arrays (with old timesteps)
	/************************************/
	//free(mass_Lys_orig);
	//free(mass_SW_orig);
	free(time_orig);
	free(date_orig);
	free(tsec_orig);
	
	
	
	/************************************************
	 * Medianfilter
	/************************************************/
	printf("Medianfilter...\n");
	
	for(i=0; i<nlys; i++){
		runmedian(mass_SW[i], datapoints, median_n);
		runmedian(mass_sum[i], datapoints, median_n);		
	}
	
	
	//print results
	if(printinter){
		sprintf(filename, (char *)"output/03a_median_Sw.dat");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		
		printdatatofile(fout, tsec, mass_SW, nlys, datapoints);
		fclose(fout);
		
// 		sprintf(filename, (char *)"output/03a_median_Lys.dat");
// 		fout = fopen(filename, "w");
// 		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
// 		
// 		printdatatofile(fout, tsec, mass_Lys, nlys, datapoints);
// 		fclose(fout);
		
		sprintf(filename, (char *)"output/03a_median_sum.dat");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		
		printdatatofile(fout, tsec, mass_sum, nlys, datapoints);
		fclose(fout);
		
		//generate and print fluxfiles
		getfluxes(datapoints, nlys, mass_sum, mass_SW, timestep, topflux, bottomflux);
				
		sprintf(filename, (char *)"output/03a_median_topflux.dat");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		printdatatofile(fout, tsec, topflux, nlys, datapoints);
		fclose(fout);
		
		sprintf(filename, (char *)"output/03a_median_bottomflux.dat");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		printdatatofile(fout, tsec, bottomflux, nlys, datapoints);
		fclose(fout);
	}

	/************************************************
	 * Running mean
	/************************************************/
	printf("Runmean...\n");
	
	
	//Filter zum Aktivieren oder deaktivieren...

	for(i=0; i<nlys; i++){
		/***************************
		 * Filtering Seepage Data
		/***************************/
		runmean(mass_SW[i], datapoints, runmean_n);
	
		
		/***********************
		 * Filtering Lysimetermass (Storage)
		/***********************/
		runmean(mass_sum[i], datapoints, runmean_n);
				
		
		/*******************************
		 * alternative Filter
		/******************************/
		// runmean(mass_Lys[i], datapoints, 11); adaptedthreshold2(mass_Lys[i], datapoints, 15, 2, fthresh);
		//runmean(mass_Lys[i], datapoints, 21); peaks(mass_Lys[i], datapoints, 0.150);
		//anisodifffilter(mass_Lys[i], datapoints, 0.5, 10, 3000);
		//totvarfilter(mass_Lys[i], datapoints);
		//fourierfilter(datapoints, mass_Lys[i], 300.);
		//luulfilter(mass_Lys[i], datapoints, 121);
		//runmean(mass_Lys[i], datapoints, 21);
		//adapteddiffusion(mass_Lys[i], datapoints, diff_timestep);
		//adaptedthreshold(mass_Lys[i], datapoints);	
		//fankfilter(mass_Lys[i], datapoints, 10);
		
	}

	//print results
	if(printinter){
		sprintf(filename, (char *)"output/03b_runmean_Sw.dat");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		
		printdatatofile(fout, tsec, mass_SW, nlys, datapoints);
		fclose(fout);
		
// 		sprintf(filename, (char *)"output/03b_runmean_Lys.dat");
// 		fout = fopen(filename, "w");
// 		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
// 		
// 		printdatatofile(fout, tsec, mass_Lys, nlys, datapoints);
// 		fclose(fout);
		
		sprintf(filename, (char *)"output/03b_runmean_sum.dat");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		
		printdatatofile(fout, tsec, mass_sum, nlys, datapoints);
		fclose(fout);
		
		//generate and print fluxfiles
		getfluxes(datapoints, nlys, mass_sum, mass_SW, timestep, topflux, bottomflux);
				
		sprintf(filename, (char *)"output/03b_runmean_topflux.dat");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		printdatatofile(fout, tsec, topflux, nlys, datapoints);
		fclose(fout);
		
		sprintf(filename, (char *)"output/03b_runmean_bottomflux.dat");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		printdatatofile(fout, tsec, bottomflux, nlys, datapoints);
		fclose(fout);
	}
	
	/***************************************
	 * Oscillation Threshold-Filter
	/***************************************/		
	if(synchro==0){
		printf("Noisethreshold with fixed value (standard method)...\n");
		for(i=0; i<nlys; i++)
			noisethreshold(mass_sum[i], datapoints, noisethresh);
	}
	
	else if(synchro==1){
		printf("Adaptive noisethreshold, using parallel lysimeters...\n");
		synchrothreshold(mass_sum, datapoints, nlys, thresh);

	}
	else if(synchro==2){
		printf("Adaptive noisethreshold, using parallel lysimeters with a reduced parallelity of only 5 lysimeters...\n");
		synchrothreshold_fiveLysimeters(mass_sum, datapoints, nlys, thresh);

	}
	



	//print results
	if(1){
		sprintf(filename, (char *)"output/Filtered_01_Sw_mass.dat");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		
		printdatatofile(fout, tsec, mass_SW, nlys, datapoints);
		fclose(fout);
		
		//sprintf(filename, (char *)"output/Filtered_01B_Lys_mass.dat");
		//fout = fopen(filename, "w");
		//if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		
		//printdatatofile(fout, tsec, mass_Lys, nlys, datapoints);
		//fclose(fout);
		
		sprintf(filename, (char *)"output/Filtered_02_masssum.dat");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		
		printdatatofile(fout, tsec, mass_sum, nlys, datapoints);
		fclose(fout);
		
		//generate and print fluxfiles
		getfluxes(datapoints, nlys, mass_sum, mass_SW, timestep, topflux, bottomflux);
				
		sprintf(filename, (char *)"output/Filtered_03_topflux.dat");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		printdatatofile(fout, tsec, topflux, nlys, datapoints);
		fclose(fout);
		
		sprintf(filename, (char *)"output/Filtered_04_bottomflux.dat");
		fout = fopen(filename, "w");
		if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
		printdatatofile(fout, tsec, bottomflux, nlys, datapoints);
		fclose(fout);
	}
	
	
	/************************************
	 * CALCULATE WATERBALANCE
	/************************************/
	printf("Calculate cumulative waterbalance...\n");
	getETPSS(datapoints, nlys, mass_sum, mass_SW, evapo, precip, storage, seepage);
	
	////////////
	//print data
	////////////
	
	//open file
	sprintf(filename, (char *)"output/waterbalance.dat");
	fout = fopen(filename, "w");
	if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
	
	//print header
	fprintf(fout, "date");
	for(i=0; i<nlys; i++){
		fprintf(fout, "\tET_L%d(mm)\tP_L%d(mm)\tStorage_L%d(mm)\tseepage_L%d(mm)", i+1, i+1, i+1, i+1);
	}
	fprintf(fout, "\n");
	
	//printdata
	for(l=0; l<datapoints; l++){	
		fprintf(fout, "%s", date[l]);
		for(i=0; i<nlys; i++)
			fprintf(fout, "\t%lf\t%lf\t%lf\t%lf", evapo[i][l], precip[i][l], storage[i][l], seepage[i][l]);
		fprintf(fout, "\n");
	}

	fclose(fout);

	sprintf(filename, (char *)"output/waterbalance_end_values.dat");
	fout = fopen(filename, "w");
	if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
	
	//print results
	fprintf(fout, "waterbalance for time period from %s to %s\n\n", date[0], date[datapoints-1]);
	for(i=0; i<nlys; i++){
		fprintf(fout, "Lysimeter_%d:\n", i+1);
		fprintf(fout, "Precipitation  (P): %.1lf\n", precip[i][datapoints-1]);
		fprintf(fout, "Evapotranspir.(ET): %.1lf\n", evapo[i][datapoints-1]);
		fprintf(fout, "Storage       (St): %.1lf\n", storage[i][datapoints-1]);
		fprintf(fout, "Seepagewater  (Sw): %.1lf\n\n", seepage[i][datapoints-1]);
	}
		
	fclose(fout);
	
	/************************************
	 * CALCULATE Daily Precipitation
	/************************************/
	printf("Calculate daily precipitation...\n");
	
	//open file
	sprintf(filename, (char *)"output/dailyprecipitation.dat");
	fout = fopen(filename, "w");
	if(fout==NULL) { printf("\n\nDatei %s kann nicht geffnet werden!\n\n", filename); exit(1);}
	
	getdailyprecip(datapoints, nlys, date, precip, fout);

	//fclose(fout2);
	
	
	
	

	return 0;
}

time_t readtime(char *buffer, struct tm *myTm)
{
	char delimiter[] = "/.:,_", *ptr;
	
		
		ptr = strtok(buffer, delimiter); myTm->tm_year=(atoi(ptr)-1900);//Jahr
		//printf("\nJahr: %s  %d",ptr,atoi(ptr));
		ptr = strtok(NULL, delimiter); myTm->tm_mon=(atoi(ptr)-1);//Monat
		//printf("\nMonat: %s  %d",ptr,atoi(ptr));
		ptr = strtok(NULL, delimiter); myTm->tm_mday=atoi(ptr);//Tag
		//printf("\nTag: %s  %d",ptr,atoi(ptr));
		ptr = strtok(NULL, delimiter); myTm->tm_hour=atoi(ptr);//Stunde
		//printf("\nStunde: %s  %d",ptr,atoi(ptr));		
		ptr = strtok(NULL, delimiter); myTm->tm_min=atoi(ptr); //Minute
		//printf("\nMinute: %s  %d",ptr,atoi(ptr));
		ptr = strtok(NULL, delimiter); myTm->tm_sec=atoi(ptr); //Sekunde
		//printf("\nMinute: %s  %d",ptr,atoi(ptr));
		
		//summertime-flag is set to 1 - d.h. Sommerzeit-Korrektur wird nicht durchgefìhrt
		myTm->tm_isdst=0;
	
	return mktime(myTm);
}

/****************************************************
 * calculates cumulative fluxes of ET, Precipitation, Storage, Seepage out of the mass-data
 * *************************************************/

void getETPSS(long lines, int cols, double **mass_sum, double **mass_SW, double **evapo, double **precip, double **storage, double **seepage)
{
	int i;
	long l;
	double ETP, dstorage, dseepage, devapo, dprecip; //ETP: mass change over the upper boundary, may be ET(evapotrans) or P (precipitation)
	
	/************************************************
	* set cumulative fluxes to zero at at the beginning
	*************************************************/
	for(i=0; i<cols; i++)
		for(l=0; l<lines; l++)
			evapo[i][l]=precip[i][l]=storage[i][l]=seepage[i][l]=0.0;
	
	/*************************************************
	* LOOP OVER ALL DATA COLUMNS (different lysimeters)
	*************************************************/
	
	for(i=0; i<cols; i++){		
			
		//set first value to zero, cause there is no change to calculate
		l=0;
		storage[i][l]=seepage[i][l]=precip[i][l]=evapo[i][l]=0.0;
		
		
		//calculate waterbalance values, calculate change between l and l-1th value
		for(l=1; l<lines; l++){
			//calculate mass changes
			dseepage = mass_SW[i][l]-mass_SW[i][l-1];
			ETP = mass_sum[i][l]-mass_sum[i][l-1];
			dstorage = ETP - dseepage; //Aenderung im Lysimeter = Aenderung im Gesamtsystem - Aenderung im SW-Tank
			
			//change units from kg to mm
			dstorage = dstorage/0.9982;
			dseepage = dseepage/0.9982;
			ETP = ETP/0.9982;
			
			//decide between precipitation and ET
			if(ETP>=0.0) {dprecip=ETP; devapo=0.0;}
			else {devapo=(-1)*ETP; dprecip=0.0;}
			
			//calculate cumulated fluxes
			storage[i][l] = storage[i][l-1] + dstorage;
			seepage[i][l] = seepage[i][l-1] + dseepage;
			precip[i][l]  = precip[i][l-1] + dprecip;
			evapo[i][l]   = evapo[i][l-1] + devapo;
		}
	}
}




/*************************************************************
 * getfluxes calculates the fluxes out of the mass-files. timestep is given in minutes, the output flux is calculated in mm/h.
/*************************************************************/
void getfluxes(long lines, int cols, double **mass_sum, double **mass_SW, double timestep, double **topflux, double **bottomflux)
{
	int i;
	long l;
	
	
	for(i=0; i<cols; i++){	
		
		//set first value to zero, cause there is no change to calculate
		l=0;
		topflux[i][l]=bottomflux[i][l]=0.0;
		
		//calculate fluxes: loop over the next values
		for(l=1; l<lines; l++){
			//calculation time from minutes to hours (/(timestep/60) = *60/timestep), and kg to mm (*1000/998.2)
			bottomflux[i][l]= (mass_SW[i][l] - mass_SW[i][l-1])*1000.0/998.2/timestep*60.0;
			topflux[i][l] = (mass_sum[i][l] - mass_sum[i][l-1])*1000.0/998.2/timestep*60.0;
		}
	}
}
	
	
void readmassdata(FILE *data, long lines, int nlys, int ntimecols, long *tsec, double **mass_Lys, double **mass_SW)
{
	double dummy;
	int i;
	char **date, buffer[256], *ptr;
	long l;
	time_t now;
	struct tm *myTm;
	
	/****************************************
	 * INITIALISATION OF THE TIMESTRUCTURE myTm
	/****************************************/
	myTm = localtime(&now);
	
	
	/****************************************
	 * ALLOCATE MEMORY FOR THE DATE
	/****************************************/
	date = chmatrix(lines, 50);
	
	/****************************************
	 * goto start of file and skip header
	/****************************************/
	fseek(data,0L,0);
	nextline(data);
	
	
	/**********************************
	* LOOP OVER ALL DATALINES
	* ********************************/
	
	for(l=0; l<lines; l++){				
		
		//read date columns, if there are two, than skip first column and save only second
		if(ntimecols==1){
			fscanf(data, "%[^\t,;\n]", date[l]); fgetc(data);
		}
		if(ntimecols==2){
			 fscanf(data, "%lf", &dummy); fgetc(data);
			 fscanf(data, "%[^\t,;\n]", date[l]); fgetc(data);
		}
		
		//read data
		for(i=0; i<nlys; i++){
			
			//1. Read seepage-data
			fscanf(data, "%[^\t,;\n]",buffer); fgetc(data);
			mass_SW[i][l]=strtod(buffer, &ptr);
			
			//check if there was no number (i.e. "NN-Notanumber") in the buffer-string
			if (ptr==buffer) mass_SW[i][l]=ERROR;
									
					
			//2. Read lysimeter-data
			
			fscanf(data, "%[^\t,;\n]",buffer);
			mass_Lys[i][l]=strtod(buffer, &ptr);
			
			//check if there was no number (i.e. "NN-Notanumber") in the buffer-string
			if (ptr==buffer) mass_Lys[i][l]=ERROR;
			
			//skip delimiter (i.e. tab), except in the last column, cause there is a newline anyway
			if(i<(nlys-1)) fgetc(data); 
		}
		
		nextline(data);
	}
	
	/**********************************
	* CALCULATE TIME IN SECONDS
	* ********************************/
	for(l=0; l<lines;l++){
		tsec[l]=readtime(date[l], myTm);
		//add 1 hour for MEZ
		tsec[l] += 3600;
	}
	
	//free memory
	free(date);
	
}

void getdailyprecip(long lines, int nlys, char **date, double **precip , FILE *fout)
{
	int i;
	long l, step;
	double value, P;
	
	
	//print header
	fprintf(fout, "date");
	for(i=0; i<nlys; i++){
		fprintf(fout, "\tPrecip_L%d", i+1);
	}
	fprintf(fout, "\n");	
	
	
	
	//lege step fest als (24*60)=1440 minutes
	step = 1440;
	
	
	//suche Anfangzeit des Tages in der Stunde und Minute 00 ist!
	l=0;
	while(strstr(date[l], "_00:00")==NULL && l<lines)
		l++;
	
	printf("    ... starting new day in line l=%ld\t%s\n",l, date[l]);
	
	
	if(l==lines){
		printf("\n\nDAILYPRECIP: KEINE TAGESANFANGSZEIT gefunden! ABBRUCH!\n\n");
		return;
	}
	
	
	while(l+step<lines){
		//printf("step=%ld",l);
		
		//fprintf(fout, "test");
		fprintf(fout, "%s", date[l]);
		
				
		for(i=0; i<nlys; i++){
			value = precip[i][l+step]-precip[i][l];
			fprintf(fout, "\t%lf", value);
		}
		
		fprintf(fout, "\n");
		
		l=l+step;
	}
	
}


//hier wird die negativen Massen gleich in NaN (not a number) ìbesetzt
int printdataNN(FILE *fp, long *tsec, double **data, long cols, long lines)
{
	double tdays;
	long l, i;
	int year, month, day, hour, min, sec;
	
	time_t now, reftime, time1, ttsec;
	struct tm *myTm;
	myTm = localtime(&now);
	struct tm refTime = {0, 0, 0, 1, 0, (2011-1900), 0, 0, 0};
	reftime = mktime(&refTime);	
	
	
	for (l=0; l<lines; l++){
				
		//calculate time in days and date
		ttsec=(time_t)tsec[l];
		myTm=gmtime(&ttsec);
		
		//read date (da sonst der time1=mktime(myTm)-Befehl die Struktur myTm verñndert!!
		year=(myTm->tm_year +1900);
		month=(myTm->tm_mon +1);
		day= myTm->tm_mday;
		hour=myTm->tm_hour;
		min=myTm->tm_min;
		sec=myTm->tm_sec;
		
		time1=mktime(myTm);
		tdays= ((double)(time1-reftime))/86400.;
		//tdays = (double)(tsec[l]-tsec[0])/3600./24.;
		
		//print days
		fprintf(fp, "%lf", tdays);
		
		//print date
		fprintf(fp, "\t%d/%02d/%02d_%02d:%02d:%02d",year,month, day, hour, min, sec);
		
		//print data
		for (i=0; i<cols; i++){
			if(data[i][l]>0) fprintf(fp, "\t%.5lf", data[i][l]);
			else fprintf(fp,"\tNaN");
		}
		
		//new line
		fprintf(fp, "\n");
	}
	
	return 0;
}

double findkey(FILE *param, long lines, char *keyword)
{
	long l;
	char buffer[1024];
	double value;
	
	fseek(param,0L,0);
	for (l=0; l<lines;l++){
		fscanf(param,"%s", buffer);
		if (strstr(buffer, keyword)!=NULL){
			//printf("Word found in buffer %s\n", buffer);
			fscanf(param, "%lf", &value);
			break;
		}
		
		nextline(param);
	}
	
	if (l==lines){
		printf("\n\nKEYWORD %s not found.\n FILTERING ABORTED!!!\n\n", keyword);
		exit(1);
	}
	

	
	return value;
}

int findkey_int(FILE *param, long lines, char *keyword)
{
	long l;
	char buffer[1024];
	int value;
	
	fseek(param,0L,0);
	for (l=0; l<lines;l++){
		fscanf(param,"%s", buffer);
		if (strstr(buffer, keyword)!=NULL){
			//printf("Word found in buffer %s\n", buffer);
			fscanf(param, "%d", &value);
			break;
		}
		
		nextline(param);
	}
	
	if (l==lines){
		printf("\n\nKEYWORD %s not found.\n FILTERING ABORTED!!!\n\n", keyword);
		exit(1);
	}
	
	
	return value;
}